﻿#ifndef SYSUSERMGT
#define SYSUSERMGT

#include <topcore/topclassabs.h>
#include <tdatabaseutil/tsqlselectorv2.h>
#include <tbaseutil/tdataresponse.h>
#include <topcore/topclasshelper.h>

class TCategoryTreeView;
class TTableView;
class TSearchEntry;
class TPageTool;
class TSplitter;

class SysUserMgt : public TopClassAbs
{
    Q_OBJECT
public:
    explicit SysUserMgt(const QString &iModuleNameStr = QString(""),
                        const QVariantMap &iUrlPars = QVariantMap(),
                        QWidget *iParent = nullptr);
    ~SysUserMgt();
public slots:
    void refresh(bool iResetPageBol = true);                //刷新表格信息
    void refreshDetail(const QString &iUid);                //刷新右侧详细信息
    void reload();                                          //详细表单刷新后重新载入数据
    bool DetailModified() const;                            //用于判断详细表单的数据是否发生改变
    void setIsDetailModified(bool iIsDetailModified);       //设置详细表单的状态(是否发生改变)
    void newItem();                                         //"新建"功能
    void saveItem();                                         //保存用户信息，对应action-保存
    void removeUser(const QVariantList &iItems);            //删除表格里的用户,对应右键菜单里的action-删除
    void setDetailUid(const QString &iUid, bool iForceRefresh = false); //设置条目的uid
    void setData(const QVariantMap &iDataMap);
    QVariantList selectedItems();                           //返回选中的表格数据LIST
    QString detailUid() const;                              //返回当前条目的uid
private slots:
    void onPageChanged();                                   //翻页对应的槽函数
    void onSelectionChanged();                              //当表单中选择改变时对应的槽函数
    void onDetailSaved(const QVariant &iUidStr);            //保存详细表单时对应的槽函数
    void onDetailChanged(bool iIsDetailModified);           //详细表单内容改变时的槽函数
    void onDetailDataChanged();
signals:
    void isDetailModifiedChanged(bool iIsDetailModified);   //详细表单发生改变时发射该信号
    void detailUidChanged(const QString &iDetailUid);       //当左侧选中条目改变时详细表单的UID变化，发送该信号刷新详细表单
    void detailSaved(const QVariant &iUid);                  //当详细表单数据保存时发送
    void selectionChanged();                                //左侧表单选中条目变化时发送该信号
private:
    void initTableView();                                   //初始化界面
private:
    TSplitter *mBodySplitter = nullptr;
    TSplitter *mBodyWidget = nullptr;
    TTableView* mTableView = nullptr;
    TSearchEntry* mSearchEntry = nullptr;
    TPageTool* mPageTool = nullptr;
    TUiLoader *mDetailView;
    QWidget *mBodyWgt = nullptr;
    QString mLastDetailUid;
    QString mDetailUid;
    bool IsDetailchange = false;
    TopClassTableConf mTableConf;
    QVariantList mSelectedItems;
};

#endif //SYSUSERMGT
